import os
import json
import datetime
from bottle import route, run, template
import uuid
import time

class SchoolActions():

    def __init__(self, args):
        self.args = args

    # --------------------------------------------------------------------------

    def check_command(self, cmd):
        if cmd in self.args:
            return self.args[cmd] != ""
        return False

    # --------------------------------------------------------------------------

    def to_file(self, name):
        return (name.lower().replace(" ", "-"))+'.json'

    # --------------------------------------------------------------------------

    def create(self, name):
        if not os.path.isfile(self.to_file(name)):
            f = open(self.to_file(name), 'w')
            f.write("[]")
            f.close()
        else:
            print(f"school error: {self.to_file(name)} is already exists.")

    # --------------------------------------------------------------------------

    def add(self, name):
        if os.path.isfile(self.to_file(name)):
            struct = []

            f = open(self.to_file(name), "r")

            title = input("Title of task ? ")
            subject = input("School subject ? ")

            try:
                date_ = input("Date (format YYYY/MM/DD) ? ")
                datetime.datetime.strptime(date_, '%Y/%m/%d')
                newd = date_.split("/")
               
                d = datetime.date(int(newd[0]), int(newd[1]), int(newd[2]))
                date_ = d.strftime("%A %d %B %Y")


                hour_ = input("Hour (format HH/MM) ? ")
                datetime.datetime.strptime(hour_, '%H:%M')
            except ValueError:
                raise ValueError("school.py error: Incorrect date format.")


            content = {"id": str(uuid.uuid4()),"title": title, "subject": subject, "date": date_, "hour": hour_, "removed": "False"}

            struct = json.load(f)
            struct.append(content)
            f.close()

            
            f2 = open(self.to_file(name), "w")
            f2.write(json.dumps(struct))
            f2.close()
            

        else:
            print(f"school.py error: {self.to_file(name)} not found.")


    # --------------------------------------------------------------------------
    
    def done(self, id_, filename):
        if os.path.isfile(self.to_file(filename)):

            f_content = (open(self.to_file(filename), "r").read()).replace("true", "True").replace("false", "False")
            content = list(eval(f_content))

            for x in content:
                if x['id'] == id_:
                    x["removed"] = True
                    
            f = open(self.to_file(filename), "w")
            f.write(json.dumps(content))
            f.close()


        else:
            print(f"school error: {self.to_file(filename)} not found.")

    # --------------------------------------------------------------------------

        
    def html_view(self, title, date, hour, subject, id_):
        if datetime.datetime.strptime(date+" "+hour, "%A %d %B %Y %H:%M") < datetime.datetime.now():
            return f"""<div style="padding: 40px"><h5 style="font-family: 'Roboto Condensed', sans-serif;color: grey;"><span class="glyphicon glyphicon-time"></span> {date} {hour} | <span class="glyphicon glyphicon-book"></span> {subject} | <span style="color: black">ID: {id_}</span></h5>
            <div style="margin-top: -12px;border-bottom: 1px solid lightgrey">
            
            <div style="margin-top: 25px">
                <div class="alert alert-danger">
                    Task late: Unfinished task
                </div>
            </div>

            <h3 style="font-family: 'Baloo Chettan 2', cursive;">{title}</h3>
            </div>
            </div>

            <br />
            """
        else:
            return f"""<div style="padding: 40px"><h5 style="font-family: 'Roboto Condensed', sans-serif;color: grey;"><span class="glyphicon glyphicon-time"></span> {date} {hour} | <span class="glyphicon glyphicon-book"></span> {subject} | <span style="color: black">ID: {id_}</span></h5>
            <div style="margin-top: -12px;border-bottom: 1px solid lightgrey">
            <h3 style="font-family: 'Baloo Chettan 2', cursive;">{title}</h3>
            </div>
            </div>

            <br />
            """
        

    # --------------------------------------------------------------------------    

    def serve(self, filename, port="4000"):
        if os.path.isfile(self.to_file(filename)):

            f_content = (open(self.to_file(filename), "r").read()).replace("true", "True").replace("false", "False")
            content = sorted(list(eval(f_content)),key=lambda x : time.strptime(x['date'],"%A %d %B %Y"))
            
            if port.isnumeric():
                
                pretty_print = """"""
           
                for x in content:
                    if x["removed"] != True:
                        pretty_print += self.html_view(x["title"], x["date"], x["hour"], x["subject"], x["id"])
                    
          
                @route('/')
                def hello():
                    return f"""<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8" />
<title>{filename.title()}</title>
<link rel="icon" type="image/png" href="https://raw.githubusercontent.com/mortim1/school/master/assets/favicon.png" />
<link href="https://fonts.googleapis.com/css?family=Sen&display=swap" rel="stylesheet">
<link href="https://fonts.googleapis.com/css?family=Baloo+Chettan+2&display=swap" rel="stylesheet">
<link href="https://fonts.googleapis.com/css?family=Roboto+Condensed&display=swap" rel="stylesheet">
<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css" integrity="sha384-BVYiiSIFeK1dGmJRAkycuHAHRg32OmUcww7on3RYdg4Va+PmSTsz/K68vbdEjh4u" crossorigin="anonymous"> 
<meta name="viewport" content="width=device-width, initial-scale=1">
</head>
<body>
    <h1 style="font-family: 'Sen', sans-serif;border-bottom: 1px solid lightgrey;padding: 15px"><span class="glyphicon glyphicon-education" style="font-size: 28px"></span> {filename.title()}</h1>
    {pretty_print}
</body>
</html>
"""

                run(host='localhost', port=port)
            else:
                print("school error: --port argument is not a decimal.")

        else:
            print(f"school error: {self.to_file(filename)} not found.")


    # --------------------------------------------------------------------------

    def restore(self, id_, name):
        if os.path.isfile(self.to_file(name)):
            f_content = (open(self.to_file(name), "r").read()).replace("true", "True").replace("false", "False")
            content = eval(f_content)
            
            for x in content:
                if x["id"] == id_:
                    x["removed"] = False

        
            f = open(self.to_file(name), "w")
            f.write(json.dumps(content))
            f.close()

        else:
            print(f"school error: {self.to_file(name)} not found.")

    # --------------------------------------------------------------------------

    def run(self):
        
        if self.check_command("create"):
            return self.create(self.args['create'])

        elif self.check_command("add"):
            return self.add(self.args['add'])

        elif "id" in self.args and "name" in self.args:
            return self.done(self.args["id"], self.args["name"])

        elif "restore_id" in self.args and "restore_name" in self.args:
            return self.restore(self.args["restore_id"], self.args["restore_name"])

        elif "name" in self.args and "port" in self.args:
            return self.serve(self.args["name"], self.args["port"]) if self.args["port"] != None else self.serve(self.args["name"])